@@ -2,6 +2,7 @@
 Transfer an object to its gear, as an interface.
 '''
 import asyncio
+import datetime
 
 from .AsyncPeriod import AsyncPeriod
 
@@ -21,6 +22,7 @@ class _Gear:
         self._unlocked.set()
         self.assistant_tasks = []
         self.prev_period = None
+        self._current_period: AsyncPeriod = None
 
     def delete(self):
         '''
@@ -56,10 +58,10 @@ class _Gear:
             if not bool(call_backs[attr]):
                 call_backs.pop(attr)
 
-        for attr in set(getattr(type(self.obj), '__dict__', {}).values()) | \
-                    set(getattr(self.obj, '__dict__', {}).values()):  # 遍历绑定对象的命名空间及其类命名空间，找到实例回调方法对应的类函数
+        for attr in list(getattr(type(self.obj), '__dict__', {}).values()) + \
+                    list(getattr(self.obj, '__dict__', {}).values()):  # 遍历绑定对象的命名空间及其类命名空间，找到实例回调方法对应的类函数
             attr = getattr(attr, '__func__', attr) if type(attr) is classmethod else attr
-            if attr in call_backs:
+            if attr in list(call_backs.keys()):
                 _set_intance_gear_callbacks(attr, self.obj, period_names)
 
     def add_periods(self, *new_period_names: str):
@@ -83,9 +85,17 @@ class _Gear:
 
         :return:
         '''
-        for name, period in self.periods.items():
-            if period.get_state():
-                return name
+        if self._current_period is not None:
+            return self._current_period._name
+
+    def current_set_datetime(self) -> datetime.datetime:
+        '''
+        Get the UTC datetime when the present period is set.
+
+        :return:
+        '''
+        if self._current_period is not None:
+            return self._current_period._ensured_time
 
     def get_period_names(self):
         '''
@@ -95,6 +105,21 @@ class _Gear:
         '''
         return tuple(self.periods.keys())
 
+    def sync_set_period(self, period_name: str, slot_num: int = 1):
+        '''
+        Synchronous version of set_period.
+        Set obj to period period_name when unlocked, otherwise PermissionError is raised.
+
+        :param period_name:
+        :param slot_num: Attention! Do not use it if you do not understand the parameter!
+                slot_num means that only after slot_num times Gear(obj).set_period(period_name,slot_num) run,
+                the period of Gear(obj) could really be set to period_name, which is interrupted
+                if among these times set_period run, the same period_name with a different slot_num is given.
+                Then the procedure is refreshed, the count would be reset.
+        :return:
+        '''
+        return self._set_period(period_name, slot_num)
+
     def _set_period(self, period_name: str, slot_num: int = 1):
         p = self.periods[period_name]
         p.slots_num_for_true = slot_num
